* 
* Test.asm ---  Test program for the Ep2711E9 Rev.C board
*		(c)2003, EVBplus.com, Written by Wayne Chu
*
*     Function:	This is the factory test program for checking on-board
*		hardware. It checks the DIP switch of the port C, 8 LEDs of
*		the port B, the pusubuttons PC0,PC1,PA0, and the LCD display.
*		It scans the keypad and displays the key number on the 7 
*		segment LED display while playing a song. It allows to 
*		adjust the trimmer pot to change display's brightness.
*
*     Instructions:
*	   If you have a 4 x 4 keypad, connect it to the keypad header J6.
*	   Before running the test program, place the all individual DIP 
*	   switches at the upper positions.
*
*	1. After running the test program, test each individual switch on
*	   the DIP switch and see the corresponding LED status on the PB0-PB7.
*	   	LCD display shows:	"PRESS PA0 SWITCH"
*                                       "TO CONTINUE TEST"
*
*	2. Test the pushbutton switches PC0 and PC1 only when the 
*	   PC0 and PC1 switches on the DIP switch are in the upper position.
*		LCD display shows:	"PRESS PA0 SWITCH"
*                                       "TO CONTINUE TEST"
*
*	3. Press the pushbutton switch PA0 and the music should come out.
*	   The hex number 0 to F should be shifting out on the 7 segment LED
*	   display. The RX LED should be off and the TX LED should be on.
*		LCD display shows:	"TESTING IR LIGHT"
*                                       "NO OBJECT NEARBY"
*
*	4. Adjust the trimmer pot, the 7 segment LED display's brightness 
*	   should change. If you press any key on the keypad, the 7 segment
*	   LED display will show the key number that you pressed.
*		LCD display shows:	"TESTING IR LIGHT"
*                                       "NO OBJECT NEARBY"
*
*	5. Place your hand 6" from the IR transceiver to allow the IR light
*	   reflect. The both RX LED and TX LED should be on, then observe the
*	   message on the LCD display. See IR_sen.asm for details.
*		LCD display shows:	"TESTING IR LIGHT"
*                                       " OBJECT DETECTED"
*
* The 68HC11 control register address is the offset address from the
* register base address $1000, so index addressing mode can also be used,
* otherwise only extended addressing mode can be used.
*
* 
* The only changes required to run the test program under the BUFFALO are:
* 1. User program starts at $D000, not $F000
* 2. Interrupt vectors are redirected to RAM jump table starting at $00C4
*

        ORG  $c4
*** Vector jump table for BUFFALO monitor ***
JSCI    RMB   3
JSPI    RMB   3
JPAIE   RMB   3
JPAO    RMB   3
JTOF    RMB   3
JTOC5   RMB   3
JTOC4   RMB   3
JTOC3   RMB   3
JTOC2   RMB   3
JTOC1   RMB   3
JTIC3   RMB   3
JTIC2   RMB   3
JTIC1   RMB   3
JRTI    RMB   3
JIRQ    RMB   3
JXIRQ   RMB   3
JSWI    RMB   3
JILLOP  RMB   3
JCOP    RMB   3
JCLM    RMB   3
	


portg:  equ     $6200	; port B data register of the 65C22
ddrg:   equ     $6202	; port B direction registr of the 65C22

MULTI_MODE:	equ	$10
SINGLE_MODE:	equ	0
SCAN_MODE:	equ	$20
NO_SCAN_MODE:	equ	0
CHANNEL_NUM:	equ	7		; reading input from PE7


DB0:		equ	1
DB1:		equ	2
DB2:		equ   	4
DB3:		equ   	8
DB4:		equ	$10
DB5:		equ	$20
DB6:		equ	$40
DB7:		equ	$80
NOTDB5:		equ	$DF
NOTDB6:		equ	$BF
OL4:		equ	DB2
OM4:		equ	DB3
OL3:		equ	DB4
OM3:		equ	DB5
porta:		equ	0
portb:		equ	4
portc:		equ	3
ddrc:		equ	7
portd:		equ	8
ddrd:		equ	9
toc2:		equ	$18	
tctl1:		equ	$20
toc3:		equ	$1a	
tmsk1:		equ	$22
tflg1:		equ	$23	
tmsk2:		equ	$24
sccr2:		equ	$2d	
adctl:		equ	$30	
adr1:		equ	$31	
adr2:		equ	$32	
adr3:		equ	$33	
adr4:		equ	$34	
option:		equ	$39	

REGBLK:		equ	$1000

TB5MS:		equ	10000	; time base of 10,000 instruction cycles
*				; = 10,000 x 0.5us = 5ms at 8mhz
F3500HZ:	equ	351	; 3.5KHz at 8MHz
*
	org	$00
ram:		rmb	1
temp:           rmb     1
shift_cnt:	rmb	1
select:		rmb	1
spk_tone:	rmb	2
sound_dur:	rmb	1
xsound_save:	rmb	2
sound_repeat:	rmb	1
xsound_beg:	rmb	2
sound_start:	rmb	1
rest_note:	rmb	1
d5ms_flag:	rmb	1
key_flag:	rmb	1
disptn:		rmb	4
key4:		rmb	4
xsave:		rmb	2
disp_ram:	rmb	3
adctl_image:	rmb	1
brtness:	rmb	1
turn_led_on:	rmb	1

ramend:		rmb	1
ALLRAM:		equ 	ramend-ram          ;total ram used


                org     $FFA0	; Buffalo I/O routines
upcase:         rmb     3
wchek:          rmb     3
dchek:          rmb     3
init_sci:       rmb     3
input:          rmb     3
output:         rmb     3
outlhlf:        rmb     3
outrhlf:        rmb     3
outa:           rmb     3
out1byt:        rmb     3
out1bsp:        rmb     3
out2bsp:        rmb     3
outcrlf:        rmb     3
outstrg:        rmb     3
outstrg0:	rmb	3
inchar:         rmb     3
*
STACK:		equ	$8FFF

ONE_MS		equ	167	; for 8 MHz    1000us/6us=167
FIVE_MS		equ	835
TEN_MS		equ	1670


PORTF   	equ	$6201
DDRF   		equ	$6203
d40us:		equ	15	; 15x6us= 90 us, 60us ok
REG_SEL:	equ	DB0	; 0=reg, 1=data
ENABLE:		equ     DB1
NOT_REG_SEL:	equ	$FE
NOT_ENABLE:	equ     $FD
  		org	0
temp1:		rmb	1
pfimg:		rmb	1
reset_seq:	rmb	1
*
LCDimg:		equ	pfimg
LCD_RSimg:	equ	pfimg
LCD_ENimg:	equ	pfimg

LCD:		equ	PORTF
LCD_RS:		equ	PORTF
LCD_EN:		equ	PORTF
*
	org	$D000
	jmp	start
* Binary number:	 	0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
* Converted to segment code:  	0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F
*
* Binary number:		$10,$11,$12,$13,$14,$15,$16,$17
* Converted to segment code:     G   H   h   J   L   n   o   o
*
* Binary number:  		$18,$19,$1A,$1B,$1C,$1D,$1E,$1F,$20
* Converted to segment code:     P   r   t   U   u   y   _  --  Blank
*
segm:
	fcb     $3f,$06,$5b,$4f,$66,$6d,$7d,$07		;0-7
        fcb     $7f,$6f,$77,$7c,$39,$5e,$79,$71		;8-$0f
       	fcb     $3d,$76,$74,$1e,$38,$54,$63,$5c		;10-17
*		 G,  H,  h,  J   L   n   o   o
       	fcb     $73,$50,$78,$3e,$1c,$6e,$08,$40		;18-1f
*		 P,  r,  t,  U,  u   Y   -   -
       	fcb     $00,$01,$48,$41,$09,$49			;20-23
*		blk, -,  =,  =,  =,  =
*
seven_segment:
	pshx
	pshb
	ldx	#segm
	psha
	anda	#$3f
	tab
	abx
	ldaa	0,x		; get segment
	pulb
	andb	#$80		; add DP
	aba
	pulb
	pulx
	rts
*
lcd_ini:
	ldaa	#$FF
	staa	DDRF
	clra
	staa	pfimg
	staa	PORTF
       	ldx	#inidsp1   		; point to init. codes.
	ldaa	#1
	staa	reset_seq		; need more delay for first reset seq.
       	jsr    	outins1    		; output codes.
       	ldx	#inidsp2   		; point to init. codes.
	clr	reset_seq
       	jsr    	outins2    		; output codes.
	rts
inidsp1:
	fcb	2		; number of instrutions
	fcb	$33		; reset char
	fcb	$32		; reste char
inidsp2:
	fcb	4		; number of instrutions
	fcb	$28   		; 4bit, 2 line, 5X7 dot
	fcb	$06		; cursor increment, disable display shift
	fcb	$0c		; display on, cursor off, no blinking
	fcb	$01		; clear display memory, set cursor to home pos
outins1:
	pshb            		; output instruction command.
       	jsr    	sel_inst
       	ldab   	0,x
       	inx
onext1:	ldaa   	0,x
       	jsr    	wrt_pulse    		; initiate write pulse.
       	inx
	jsr	d5ms
       	decb
       	bne    	onext1
       	pulb
       	rts
*
outins2:
	pshb            		; output instruction command.
       	jsr    	sel_inst
       	ldab   	0,x
       	inx
onext2:	ldaa   	0,x
       	jsr    	wrt_pulse    		; initiate write pulse.
       	inx
       	decb
       	bne    	onext2
	jsr	d5ms
       	pulb
       	rts
*
delay_10ms:
	nop
d10ms:  pshx
	ldx     #TEN_MS
  	bsr	del1
  	pulx
	rts
d5ms:   pshx
	ldx     #FIVE_MS
  	bsr	del1
	pulx
	rts
del1:	dex
	inx
	dex
	bne	del1
	rts
*
*
sel_data:
	psha
*	bset	LCD_RSimg REG_SEL	; select instruction
	ldaa	LCD_RSimg
	oraa	#REG_SEL
	bra	sel_i
sel_inst:
	psha
*	bclr	LCD_RSimg REG_SEL	; select instruction
	ldaa	LCD_RSimg
	anda	#NOT_REG_SEL
sel_i:	staa	LCD_RSimg
	staa	LCD_RS
	pula
        rts
*

lcd_line1:
	jsr    	sel_inst		; select instruction
       	ldaa   	#$80     		; starting address for the line1
	bra	line3
lcd_line2:
	jsr    	sel_inst
       	ldaa   	#$C0     		; starting address for the line2
line3: 	jsr    	wrt_pulse
*
       	jsr    	sel_data
	jsr	msg_out
	rts

* at entry, x must point to the begining of the message,
*           b = number of the character to be sent out
*
msg_out:
	ldaa   	0,x
       	jsr    	wrt_pulse
       	inx
       	decb
       	bne    	msg_out
	rts

*
*       @ enter, a=data to output
*
wrt_pulse:
	pshx
       	psha        			; save it tomporarily.
       	anda   #$f0     		; mask out 4 low bits.
       	staa   temp1 			; save nibble value.
       	ldaa   LCDimg    		; get LCD port image.
       	anda   #$0f     		; need low 4 bits.
       	oraa   temp1    		; add in low 4 bits.
       	staa   LCDimg    		; save it
       	staa   LCD    			; output data
*
	bsr	enable_pulse
	ldaa	reset_seq
	beq	wrtpls
	jsr	d5ms			; delay for reset sequence
*
wrtpls:	pula
 	asla            		; move low bits over.
	asla
      	asla
       	asla
       	staa   	temp1			; store temporarily.
*
       	ldaa   	LCDimg 			; get LCD port image.
       	anda   	#$0f   			; need low 4 bits.
       	oraa   	temp1  			; add in loiw 4 bits.
       	staa   	LCDimg    		; save it
       	staa   	LCD    			; output data
*
	bsr	enable_pulse
	pulx
       	rts
*
enable_pulse:
*	bset	LCD_ENimg ENABLE	; ENABLE=high
	ldaa	LCD_ENimg
	oraa	#ENABLE
	staa	LCD_ENimg
	staa	LCD_EN
*	bclr	LCD_ENimg ENABLE	; ENABLE=low
	ldaa	LCD_ENimg
	anda	#NOT_ENABLE
	staa	LCD_ENimg
	staa	LCD_EN
	pshx
	ldx     #d40us
  	jsr	del1
	pulx
	rts

BLANK_MSG:
	FCC     "                "
MSG1:   FCC     "PRESS PA0 SWITCH"
MSG2:   FCC     "TO CONTINUE TEST"

TEST_MSG:
	FCC     "TESTING IR LIGHT"
YES_MSG:
	FCC     " OBJECT DETECTED"
NO_MSG: FCC     "NO OBJECT NEARBY"

*
* to calculate note value by 1000000/freq for 8 MHz

c0	equ	30578		; 32.703 Hz at 8 MHz
c0s	equ	28862		; 34.648 Hz at 8 MHz
d0	equ	27242		; 36.708 Hz at 8 MHz
d0s	equ	25714		; 38.890 Hz at 8 MHz
e0	equ	24270		; 41.203 Hz at 8 MHz
f0	equ	22908		; 43.653 Hz at 8 MHz
f0s	equ	21622		; 46.249 Hz at 8 MHz
g0	equ	20409		; 48.999 Hz at 8 MHz
g0s	equ	19263		; 51.913 Hz at 8 MHz
a0	equ	18182		; 55.000 Hz at 8 MHz
a0s	equ	17161		; 58.270 Hz at 8 MHz
b0	equ	16198		; 61.735 Hz at 8 MHz

c1	equ	15289		; 65.406 Hz at 8 MHz
c1s	equ	14431		; 69.295 Hz at 8 MHz
d1	equ	13621		; 73.416 Hz at 8 MHz
d1s	equ	12857		; 77.781 Hz at 8 MHz
e1	equ	12135		; 82.406 Hz at 8 MHz
f1	equ	11454		; 87.307 Hz at 8 MHz
f1s	equ	10811		; 92.499 Hz at 8 MHz
g1	equ	10204		; 97.998 Hz at 8 MHz
g1s	equ	9632		; 103.82 Hz at 8 MHz
a1	equ	9091		; 110.00 Hz at 8 MHz
a1s	equ	8581		; 116.54 Hz at 8 MHz
b1	equ	8099		; 123.47 Hz at 8 MHz

c2	equ	7645		; 130.81 Hz at 8 MHz
c2s	equ	7216		; 138.59 Hz at 8 MHz
d2	equ	6811		; 146.83 Hz at 8 MHz
d2s	equ	6428		; 155.56 Hz at 8 MHz
e2	equ	6068		; 164.81 Hz at 8 MHz
f2	equ	5727		; 174.61 Hz at 8 MHz
f2s	equ	5406		; 184.99 Hz at 8 MHz
g2	equ	5102		; 195.99 Hz at 8 MHz
g2s	equ	4816		; 207.65 Hz at 8 MHz
a2	equ	4545		; 220.00 Hz at 8 MHz
a2s	equ	4290		; 233.08 Hz at 8 MHz
b2	equ	4050		; 246.94 Hz at 8 MHz

c3	equ	3822		; 261.63 Hz at 8 MHz
c3s	equ	3608		; 277.18 Hz at 8 MHz
d3	equ	3405		; 293.66 Hz at 8 MHz
d3s	equ	3214		; 311.13 Hz at 8 MHz
e3	equ	3034		; 329.63 Hz at 8 MHz
f3	equ	2863		; 349.23 Hz at 8 MHz
f3s	equ	2703		; 369.99 Hz at 8 MHz
g3	equ	2551		; 391.99 Hz at 8 MHz
g3s	equ	2408		; 415.31 Hz at 8 MHz
a3	equ	2273		; 440.00 Hz at 8 MHz
a3s	equ	2145		; 466.16 Hz at 8 MHz
b3	equ	2025		; 493.88 Hz at 8 MHz

c4	equ	1911		; 523.25 Hz at 8 MHz
c4s	equ	1804		; 554.37 Hz at 8 MHz
d4	equ	1703		; 587.33 Hz at 8 MHz
d4s	equ	1607		; 622.25 Hz at 8 MHz
e4	equ	1517		; 659.26 Hz at 8 MHz
f4	equ	1432		; 698.46 Hz at 8 MHz
f4s	equ	1351		; 739.99 Hz at 8 MHz
g4	equ	1276		; 783.99 Hz at 8 MHz
g4s	equ	1204		; 830.61 Hz at 8 MHz
a4	equ	1136		; 880.00 Hz at 8 MHz
a4s	equ	1073		; 932.32 Hz at 8 MHz
b4	equ	1012		; 987.77 Hz at 8 MHz

c5	equ	956		; 1046.50 Hz at 8 MHz
c5s	equ	902		; 1108.73 Hz at 8 MHz
d5	equ	851		; 1174.66 Hz at 8 MHz
d5s	equ	804		; 1244.51 Hz at 8 MHz
e5	equ	758		; 1318.51 Hz at 8 MHz
f5	equ	716		; 1396.91 Hz at 8 MHz
f5s	equ	676		; 1479.98 Hz at 8 MHz
g5	equ	638		; 1567.98 Hz at 8 MHz
g5s	equ	602		; 1661.22 Hz at 8 MHz
a5	equ	568		; 1760.00 Hz at 8 MHz
a5s	equ	536		; 1864.66 Hz at 8 MHz
b5	equ	506		; 1975.53 Hz at 8 MHz

c6	equ	478		; 2093.00 Hz at 8 MHz
c6s	equ	451		; 2217.46 Hz at 8 MHz
d6	equ	426		; 2349.32 Hz at 8 MHz
d6s	equ	402		; 2489.02 Hz at 8 MHz
e6	equ	379		; 2637.02 Hz at 8 MHz
f6	equ	358		; 2793.83 Hz at 8 MHz
f6s	equ	338		; 2959.96 Hz at 8 MHz
g6	equ	319		; 3135.97 Hz at 8 MHz
g6s	equ	301		; 3322.44 Hz at 8 MHz
a6	equ	284		; 3520.00 Hz at 8 MHz
a6s	equ	268		; 3729.31 Hz at 8 MHz
b6	equ	253		; 3951.07 Hz at 8 MHz

c7	equ	239		; 4186.01 Hz at 8 MHz
c7s	equ	225		; 4434.92 Hz at 8 MHz
d7	equ	213		; 4698.64 Hz at 8 MHz
d7s	equ	201		; 4978.03 Hz at 8 MHz
e7	equ	190		; 5274.04 Hz at 8 MHz
f7	equ	179		; 5587.66 Hz at 8 MHz
f7s	equ	169		; 5919.92 Hz at 8 MHz
g7	equ	159		; 6271.93 Hz at 8 MHz
g7s	equ	150		; 6644.88 Hz at 8 MHz
a7	equ	142		; 7040.00 Hz at 8 MHz
a7s	equ	134		; 7458.63 Hz at 8 MHz
b7	equ	127		; 7902.13 Hz at 8 MHz
c8	equ	1		; for rest note

NOTE_TABLE:
	fdb	c0,c0s,d0,d0s,e0,f0,f0s,g0,g0s,a0,a0s,b0
	fdb	0,0,0,0		; dummy byte
	fdb	c1,c1s,d1,d1s,e1,f1,f1s,g1,g1s,a1,a1s,b1
	fdb	0,0,0,0		; dummy byte
	fdb	c2,c2s,d2,d2s,e2,f2,f2s,g2,g2s,a2,a2s,b2
	fdb	0,0,0,0		; dummy byte
	fdb	c3,c3s,d3,d3s,e3,f3,f3s,g3,g3s,a3,a3s,b3
	fdb	0,0,0,0		; dummy byte
	fdb	c4,c4s,d4,d4s,e4,f4,f4s,g4,g4s,a4,a4s,b4
	fdb	0,0,0,0		; dummy byte
	fdb	c5,c5s,d5,d5s,e5,f5,f5s,g5,g5s,a5,a5s,b5
	fdb	0,0,0,0		; dummy byte
	fdb	c6,c6s,d6,d6s,e6,f6,f6s,g6,g6s,a6,a6s,b6
	fdb	0,0,0,0		; dummy byte
	fdb	c7,c7s,d7,d7s,e7,f7,f7s,g7,g7s,a7,a7s,b7
	fdb	0,0,0,0		; dummy byte
	fdb	c8
*
note_c	equ	0
note_cs	equ	1
note_d	equ	2
note_ds	equ	3
note_e	equ	4
note_f	equ	5
note_fs	equ	6
note_g	equ	7
note_gs	equ	8
note_a	equ	9
note_as	equ	10
note_b	equ	11

* dur18= 1/8 note,  dur14= 1/4 note,  $fe= rest_note, $ff = end of song
	
dur132	equ	12
dur116	equ	25
dur18	equ	50
dur14	equ	100	
dur12	equ	200



* this routine will read adc input on the pin pe0 and store 4 consecutive 
* readings at adr1-adr4. (single channel)
*
*
adc_conv:
	adda	#SINGLE_MODE+NO_SCAN_MODE
*
* if you want to read multi-channel input, change above statement to
*	adda	#MULTI_MODE+NO_SCAN_MODE
*
	staa	adctl_image
	ldx	#REGBLK
	jsr	conv
	rts
*	
conv:	ldaa	adctl_image	; clear ccf flag before testing it
	staa	adctl,x
not_ready:
	brclr	adctl,x $80 not_ready
	rts
*
* keypad scan
* at exit:  if a key is down, the carry bit =1 and the accumulator b
*	    holds the key number
*	    if no key is dwon the carry bit =0
*
keybrd:	
*	ldaa 	#00001111b
	ldaa 	#$0F		; pg0-pg3 are outputs, pg4-pg7 are inputs
	staa 	ddrg			
	
	ldab	#16
*	ldaa	#11110111b
	ldaa	#$F7		; pg3=low, pg0-pg2=high
	staa	temp		; save it at emp
next_row:
	staa	portg
	nop			; add delay before checking key down
	nop
	nop
	nop
	ldaa	portg
	anda	#$F0		; only read 4 MSBs pg4-pg7
	cmpa	#$F0
	bne	keyin		; a key is pressed
	subb	#4
	beq	no_keyin	; after 4 tests, accu B will be 0

	sec
	ror	temp		
	ldaa	temp
	jmp	next_row
*
no_keyin:
	clc
	rts			; no key input

keyin:	decb
	rola
	bcs	keyin
	sec
	rts

disp_data:
	fcb	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4

clrall:	pshx
	ldx	#ram
clrr	clr	0,x
	inx
	deca
	bne	clrr
	pulx
	rts
*
start:
	lds	#STACK
	ldx	#REGBLK
	clr	sccr2,x			; disable the SCI, it was enabled by BUFFALO 
*	ldaa	#11111111b		; monitor
	ldaa	#$ff			; turn off 7-segment display
	staa	portd,x
*	ldaa 	#11111110b		; 0 =input, 1 = output
	ldaa 	#$fe			; PD0=input for IR receiver
	staa 	ddrd,x			; PD1=output for IR transmitter
*
	bclr	portd,x DB1		; make PD1=0 to xmit IR light

	ldaa	#DB5+DB6 
	staa	tmsk1,x
	bset 	option,x $80	; enable adc operation
*
	ldx	#F3500HZ	; 3.5khz
	stx	spk_tone

	ldaa	#ALLRAM
	jsr	clrall
   	jsr	delay_10ms		; delay 20ms during power up
    	jsr	delay_10ms

    	jsr	lcd_ini			; initialize the LCD 
                      	
	ldx    	#MSG1			; msg1 for line1, x points to msg1
        ldab    #16                     ; send out 16 characters
    	jsr	lcd_line1
*
    	ldx    	#MSG2			; msg2 for line2, x points to msg2
        ldab    #16                     ; send out 16 characters
     	jsr	lcd_line2

	ldx	#REGBLK
back:	ldaa	portc,x
	staa 	portb,x
	brset	porta,x DB0 back
	jsr	delay_10ms
	brset	porta,x DB0 back

	ldx    	#TEST_MSG		; test msg for line1
        ldab    #16                     ; send out 16 characters
    	jsr	lcd_line1
*
    	ldx    	#BLANK_MSG		; blank msg for line2
        ldab    #16                     ; send out 16 characters
     	jsr	lcd_line2

	cli
	jsr	spk_on
	jsr	start_sound		; start sound

	ldx	#disp_data
	stx 	xsave
begin:	
	jsr	keybrd
	bcc	nokey
   	jsr	delay_10ms		; delay 20ms during power up
	jsr	keybrd
	bcc	nokey
* key down
	tba			; move key number to ACCU A
	jsr	seven_segment	; convert Accu A to segment pattern, bit 7= DP
	oraa	#$80		; to show up DP
	ldx	#REGBLK
	staa	portb,x
	bclr	portd,x 4
	bclr	portd,x 8
	bclr	portd,x $10
	bclr	portd,x $20
	jsr	delay_10ms	; turn on display only 10ms
	bset	portd,x 4
	bset	portd,x 8
	bset	portd,x $10
	bset	portd,x $20
	jmp	begin
	
nokey:	jmp	cont
	ldx	#REGBLK
	brset	portc,x DB7 pc7_high
	jsr	spk_off
	ldaa	#DB6	 
	staa	tmsk1,x
	jmp	cont
pc7_high:
	ldaa	rest_note
	bne	cont
	jsr	spk_on
	ldaa	#DB5+DB6	 
	staa	tmsk1,x
cont:
	ldaa	#CHANNEL_NUM	; set channel number before calling 
	jsr	adc_conv	
	ldaa	adr1+REGBLK
	staa	brtness
*	
	ldx	xsave
	jsr	move		; move 4 bytes of 

	ldaa	#1
	staa	turn_led_on	; turn_on_led
	jsr	sel_digit

	ldaa	brtness		; was read from adc
	beq	turn_off	; if =0, turn off display 
*				; otherwise keep on until a=0
back1:	iny			; make each loop=31 cycles = 15.5us
	dey			; 15.5 us x256 = 4ms
	iny			; leaves 1 ms for other task
	dey
	iny
	dey
	nop
	deca
	bne	back1		
turn_off:
	clr	turn_led_on
	dec	select
	jsr	sel_digit

	inc	shift_cnt
	ldaa	shift_cnt
	cmpa	#100			; 500 ms
	bne	wait
	clr	shift_cnt
	ldx	xsave
	inx
	cpx	#disp_data+16
	bne	beg1
	ldx	#disp_data
beg1:	stx	xsave
                      	
	ldaa	portd+$1000
	rora				; rotate PD0 into carry bit
	bcs	no_IR_light
    	ldx    	#YES_MSG		; yes_msg for line1
        ldab    #16                     ; send out 16 characters
     	jsr	lcd_line2
     	jmp	wait
no_IR_light:
    	ldx    	#NO_MSG			; no_msg for line1
        ldab    #16                     ; send out 16 characters
    	jsr	lcd_line2

*
*
wait:	tst	d5ms_flag
	beq	wait
	clr	d5ms_flag
	jmp	begin
*
*    this routine moves 4 bytes of data into display
*    pattern and converts the pattern to seven segment code.
*    @ enter, x points the source address
*
move:	ldy	#disptn
mnext: 	ldaa	0,x
	jsr	seven_segment	; convert Accu A to segment pattern, bit 7= DP
	staa	0,y
	inx
	iny
	cpy	#disptn+4
        bne	mnext
	rts
*
* multiplexing display one digit at a time
*
sel_digit:
	ldx	#REGBLK
	inc	select
	ldab	select
	andb	#3
	tstb
	beq	digit3
	decb
	beq	digit2
	decb
	beq	digit1
*
digit0: 
	ldaa	disptn+3
	staa	portb,x
	tst	turn_led_on
	bne	dig0_on
	clr	portb,x
dig0_on:
	bclr	portd,x 4
	bset	portd,x 8
	bset	portd,x $10
	bset	portd,x $20
	rts
*
digit1: 
	ldaa	disptn+2
	staa	portb,x
	tst	turn_led_on
	bne	dig1_on
	clr	portb,x
dig1_on:
	bset	portd,x 4
	bclr	portd,x 8
	bset	portd,x $10
	bset	portd,x $20
	rts
*
digit2: 
	ldaa	disptn+1
	staa	portb,x
	tst	turn_led_on
	bne	dig2_on
	clr	portb,x
dig2_on:
	bset	portd,x 4
	bset	portd,x 8
	bclr	portd,x $10
	bset	portd,x $20
	rts
*
digit3: 
	ldaa	disptn
	staa	portb,x
	tst	turn_led_on
	bne	dig3_on
	clr	portb,x
dig3_on:
	bset	portd,x 4
	bset	portd,x 8
	bset	portd,x $10
	bclr	portd,x $20
	rts

*
spk_on:	
	pshx
	ldx	#REGBLK
	bclr 	tctl1,x OM3
	bset 	tctl1,x OL3		; toggle speaker
	pulx
	rts
spk_off:
	pshx
	ldx	#REGBLK
	bset 	tctl1,x OM3
	bclr 	tctl1,x OL3		; turn off speaker
	pulx
	rts
*
tmr6oc2:
	ldaa	sound_start
	beq	tmr3
	ldaa	sound_dur		; duration
	deca
	staa	sound_dur
	bne	tmr3
	ldx	xsound_save
repeat:	ldab	0,x
	cmpb	#255
	beq	sound_end
	ldaa	1,x
	cmpa	#255
	beq	sound_end
	staa	sound_dur
	inx
	inx
	stx	xsound_save
	cmpb	#$fe
	bne	not_rest
	ldaa	#1
	staa	rest_note
	jsr	spk_off
	jmp	tmr3
not_rest:
	clr	rest_note
	jsr	spk_on
	ldx	#NOTE_TABLE
	aslb
	abx
	ldx	0,x
	stx	spk_tone
	jmp	tmr3	
sound_end:
	ldaa	sound_repeat
	beq	no_rep
	ldx	xsound_beg
	jmp	repeat

no_rep:	ldx	#F3500HZ	; 3.5khz
	stx	spk_tone
	jsr	spk_off
	clr	sound_start

tmr3:	ldx	#REGBLK		; in interrupt servicing routine
* in an interrupt servicing routine the x register will be saved automatically
* the rti instruction will pop the x register off stack.
	inc	d5ms_flag
	ldd	#TB5MS		; reload the count for 5 ms time base
	addd	toc2,x
	std	toc2,x
	bclr	tflg1,x NOTDB6
	rti
*
spk5oc3:
	ldx	#REGBLK		; in interrupt servicing routine
	ldd	spk_tone
	addd	toc3,x
	std	toc3,x
	bclr	tflg1,x NOTDB5
	rti
*
*
start_sound:
	ldx	#SONG
	stx	xsound_beg
	ldaa	#1
	staa	sound_repeat
	ldab	0,x
	ldaa	1,x
	staa	sound_dur
	inx
	inx
	stx	xsound_save

	ldx	#NOTE_TABLE
	aslb
	abx
	ldx	0,x
	stx	spk_tone
	ldaa	#1
	staa	sound_start
	rts

SONG:	fcb	$50+note_e,dur18
	fcb	$50+note_ds,dur18
	fcb	$50+note_e,dur18
	fcb	$50+note_ds,dur18
	fcb	$50+note_e,dur18
	fcb	$40+note_b,dur18
	fcb	$50+note_d,dur18
	fcb	$50+note_c,dur18
	fcb	$40+note_a,dur14
*	fcb	$fe,dur18
*	fcb	255,255

	fcb	$30+note_e,dur18
	fcb	$30+note_a,dur18
	fcb	$40+note_c,dur18
	fcb	$40+note_e,dur18
	fcb	$40+note_a,dur18
	fcb	$40+note_b,dur14
	fcb	$30+note_gs,dur18
	fcb	$40+note_d,dur18
	fcb	$40+note_e,dur18
	fcb	$40+note_gs,dur18
	fcb	$40+note_b,dur18
	fcb	$50+note_c,dur14

	fcb	$30+note_e,dur18
	fcb	$30+note_a,dur18
	fcb	$40+note_e,dur18
	fcb	$fe,dur14
	fcb	255,255

	
	org	JTOC3		; BUFFALO RAM vector for OC3 ($FFE4)
	jmp	spk5oc3		; jmp to OC3 service routine
	org	JTOC2		; BUFFALO RAM vector for OC2 ($FFE6)
	jmp	tmr6oc2		; jmp to OC2 service routine

	end
