*
* Tutor2.asm -- Tutorial program 2 for the Ep2711E9 Rev. C board
*               (c)2003, EVBplus.com, Written By Wayne Chu
*
*     Function:	A simple program that will flash the PB7 LED when the PA0 
*		data switch is down.  It shows how to use a subroutine call
*		and change the contents of the RAM location \fls_rate to
*		vary the flash rate.
*
*     Instruction: 
*		1. Use the B command to set a breakpoint at the address
*		   \pa0_low.
*		2. Enter the "go \start" command to run the program
*		   and nothing should happen. Then press and hold down the 
*		   data switch pa0, the program should stopped at the
*		   address \pa0_low, because the pa0 is normally high,
*		   it becomes low when it's down.
*		3. Remove the breakpoint address \pa0_low by using the Kill
*		   command, such as K \pa0_low. After the breakpoint is
*		   removed, then enter go command to run the program again.
*               4. At this time, the PB7 LED should flash when the pa0 switch
*		   is down.
*		5. Stop the program by the ESC key and use the E command to
*		   change to value of the \fls_rate from $8B to $FF,
*		   such as E \fls_rate.
*		6. Run the program by entering go command and the LED should
*		   flash at a slower rate when the pa switch is down.
*		7. Repeat the steps 5 and 6 to change the fls_rate from 
*		   $FF to $40, and the LED should flash at a faster rate.
*
*
STACK:  	equ     $FF     ; STACK at top of data ram 
FLS_CONST: 	equ     25000   ; delay 100 ms, change this number
*                               ; will change LED flash rate
*				; 25000 x 8 cycles = 200,000 cycles = 100ms
*
PA0:		equ	1
PB7:		equ	$80
porta:		equ	0
portb:		equ	4
*
REGBLK: 	equ   	$1000
*
        	org     $40     ; data ram starts at address $40
*
fls_rate:	rmb     2	; 2 byte temp storage
*
        org     $F000           ; program code starts here
*
	
start:  lds     #STACK          ; initialize the STACK point
        ldy     #FLS_CONST      ; y register = LED FLS_CONST
        sty     fls_rate        ; save FLS_CONST at fls_rate

        ldx     #REGBLK         	; x register points to register block
begin:	brclr	porta,x PA0 pa0_low	; pa0=low when pa0 switch is down
	bclr	portb,x PB7
	jmp	begin

pa0_low:
	clr	portb,x		; turn off LED PB7
        jsr     delay100ms      ; keep LED off for 100 ms
	ldaa	#PB7
	staa	portb,x         ; turn on LED
        jsr     delay100ms      ; keep LED lit for 100 ms
	jmp	begin


delay100ms:
	pshx
	ldx     fls_rate 	; re-store the y register with FLS_CONST
*
* the delay loop is 8 cycles long = 4us at 8 MHz crystal, 2 MHz bus speed
dly:    dex			; 3 cycles
	nop			; 2 cycle
        bne     dly             ; 3 cylces
	pulx
        rts                     ; 100ms delay is done

	org	$FFFE
	fdb	start
	end



